%
%%  BADH
%%  
%%  From:
%%  H. Krawczyk, SIGMA: the `SIGn-and-MAc' Approach to Authenticated
%%  Diffie-Hellman and its Use in the IDE Protocols, CRYPTO'03
%%


% Functors:

    % cryptographic primitives
    fun sig/2.
    fun enc/2.
    fun mac/2.

    % participant names 
    fun a/0.
    fun b/0.
    fun c/0.

    % constants
    fun g/0.
    fun sec/0.

    % nonces and keys
    fun rx/2.   % rx(X,Y) -- a nonce (exponent) generated by X as an initiator in the session with Y.
    fun ry/2.   % ry(X,Y) -- a nonce (exponent) generated by X as a responder in the session with Y.
    fun k/1.    % k(K) -- private key of X.
    fun pub/1.

    fun completedA/3.
    fun completedB/3.


%%%%% AUXILIARY PREDICATES %%%%%%%%%%%%

participant(a).
participant(b).
participant(c).

%%
% The following predicates specify which session will be
% taken into consideration.

% roleA(X,Y)  ---  X as an initiator runs the protocol with Y as responder 
% roleB(X,Y)  ---  X as a responder runs the protocol with Y as initiator 

roleA(a,a).
% roleA(a, b).
roleA(a, c).
% roleA(b, a).
% roleA(b, c).

roleB(a,a).
% roleB(a, b).
roleB(a, c).
% roleB(b, a).
% roleB(b, c).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The secret:

    query sec.

% Intruder rules

    rule:: sig(_,M) -> M.
    rule:: sig(K,_) -> pub(K).
    rule:: K, M -> sig(K,M).
    rule:: K, M -> enc(K,M).
    rule:: K, enc(K,M) -> M.
    rule:: K, M -> mac(K,M).
    rule:: mac(_,M) -> M.

% Initial intruder knowledge

    rule:: X             :- participant(X).
    rule:: pub(k(X))     :- participant(X).
    rule:: k(c).
    rule:: rx(c,X)       :- participant(X).
    rule:: ry(c,X)       :- participant(X).
    rule:: g.

% Protocol Rules (the main session):

    % A->B:
    %
    rule::  
        g^rx(A,B)
    :- 
        roleA(A,B).

    % B->A:
    rule::  
        GX ->  g^ry(B,A), B, sig(k(B),(GX,g^ry(B,A)))
    :- 
        roleB(B,A).

    % A->B:
    rule::  
        GY, B, sig(k(B),(g^rx(A,B), GY))
            -> A, sig(k(A),(GY,g^rx(A,B))), completedA(A,B,GY^rx(A,B))
    :- 
        roleA(A,B).

    % B comleted:
    rule:: 
        A, sig(k(A),(g^ry(B,A),GX)) -> completedB(B,A,GX^ry(B,A))
    :- 
        roleB(B,A).

%%%%%%
    
    %%%
    %% should not fire
    %
    
    rule::  K, completedB(a,a,K) -> sec.
    % rule::  K, completedB(a,b,K) -> sec.  
    % rule::  K, completedB(b,a,K) -> sec.  
    rule::  K, completedA(a,a,K) -> sec.
    % rule::  K, completedA(a,b,K) -> sec.  
    % rule::  K, completedA(b,a,K) -> sec.  
    
    % rule:: g^rx(a,b)^ry(b,a) -> sec.
    % rule:: g^rx(b,a)^ry(a,b) -> sec.
    rule:: g^rx(a,a)^ry(a,a) -> sec.

    %%%
    %% should not fire, but fire
    %
    rule:: completedA(a,c,S), completedB(a,a,S) -> sec.
    
    %%%
    %% should fire
    %
    
    % rule:: completedA(a,a,S), completedB(a,a,S) -> sec.
    % rule::  K, completedA(a,c,K) -> sec.  
    % rule::  K, completedB(a,c,K) -> sec.  
    

