%%
%%  IKEv2-DSfix
%%  
%%  IKEv2-DS fixed
%%


% Functors:

    % cryptographic primitives
    fun sig/2.
    fun enc/2.
    fun f/1.

    % participant names 
    fun a/0.
    fun b/0.
    fun c/0.

    % constants
    fun g/0.
    fun sec/0.

    % nonces and keys
    fun k/1.    % k(K) -- private key of X.
    fun pub/1.

    fun psk/2.  % psk(X,Y) -- a symmetric shared by X and Y.

    fun sa1/2.   % ...(X,Y) -- a nonce generated by X in the session with Y.
    fun sa2/2.  
    fun dhx/2. 
    fun dhy/2.
    fun ni/2.
    fun nr/2.
    fun mr/2.

    fun completedA/6.
    fun completedB/6.


%%%%% AUXILIARY PREDICATES %%%%%%%%%%%%

participant(a).
participant(b).
participant(c).

%%
% The following predicates specify which session will be
% taken into consideration.

% roleA(X,Y)  ---  X as an initiator runs the protocol with Y as responder 
% roleB(X,Y)  ---  X as a responder runs the protocol with Y as initiator 

roleA(a, a).
%roleA(a, b).
%roleA(b, a).
roleA(a, c).
%roleA(b, c).

roleB(a, a).
%roleB(a, b).
%roleB(b, a).
roleB(a, c).
%roleB(b, c).

% roleA( A, B, X, NI, SA1, SA2 )
%
roleA( A, B, dhx(A,B), ni(A,B), sa1(A,B), sa2(A,B) ) :- roleA(A,B).
    
% roleB( B, A, Y, NR )
%
roleB(B, A, dhy(B,A), nr(B,A)) :- roleB(B,A).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The secret:

    query sec.

% Intruder rules

    rule:: sig(_,M) -> M.
    rule:: sig(K,_) -> pub(K).
    rule:: K, M -> sig(K,M).
    rule:: K, M -> enc(K,M).
    rule:: K, enc(K,M) -> M.
    rule:: M -> f(M).

    % rule:: X -> km(X).
    % rule:: X -> ke(X).
    % rule:: X -> ks(X).

% Initial intruder knowledge

    rule:: X             :- participant(X).
    rule:: pub(k(X))     :- participant(X).
    rule:: k(c).
    rule:: g.

    rule:: psk(c,X)        :- participant(X).
    rule:: psk(X,c)        :- participant(X).

% Protocol Rules (the main session):


    % (1) A->B
    %
    rule::
        SA1, g^X, NI 
    :-
        roleA( _A, _B, X, NI, SA1, _SA2 ).

    % (2) B->A
    %
    rule::
        SA1, _GX, _NI
      ->
        SA1, g^Y, NR
    :-
        roleB(_B, _A, Y, NR).


    % (3) A->B
    %
    rule::
        SA1, GY, NR
      ->
        enc(SK, (A, sig(k(A), (SA1, g^X, NI, NR)), SA2))
    :-
        roleA( A, _B, X, NI, SA1, SA2 ),
        SK = f((NI, NR, SA1, GY^X)).


    % (4) B->A
    %
    rule::
        SA1, GX, NI, % from the previous rule
        enc(SK, (A, sig(k(A), (SA1, GX, NI, NR)), SA2))
      ->
        enc(SK, (B, mr(B,A), sig(k(B), (SA1, g^Y, NR, NI)), SA2))
        %           ^^^^^^^ added
        % completedB(B, A, GX, g^Y, GX^Y, SK)
    :-
        roleB(B, A, Y, NR),
        SK = f((NI, NR, SA1, GX^Y)).

    % (5) A->B (an additional step)
    %
    rule::
        NR, GY, % from the precious rule
        enc(SK, (B, MR, sig(k(B), (SA1, GY, NR, NI)), SA2))
      ->
        enc(SK, MR),
        completedA(A, B, g^X, GY, GY^X, SK)
    :-
        roleA( A, B, X, NI, SA1, SA2 ),
        SK = f((NI, NR, SA1, GY^X)).

    % (6) B completed (with an additional verification)
    %
    rule::
        SA1, GX, NI, 
        enc(SK, (A, sig(k(A), (SA1, GX, NI, NR)), _SA2)), % from previous steps
        enc(SK, mr(B,A))
      ->
        completedB(B, A, GX, g^Y, GX^Y, SK)
    :-
        roleB(B, A, Y, NR),
        SK = f((NI, NR, SA1, GX^Y)).

%%%%%%%%%%%%%%%%%%%%

    rule:: SK, completedA(a,b,_,_,_,SK) -> sec.
    rule:: SK, completedA(b,a,_,_,_,SK) -> sec.
    rule:: SK, completedB(a,b,_,_,_,SK) -> sec.
    rule:: SK, completedB(b,a,_,_,_,SK) -> sec.

    rule:: GXY, completedA(a,b,_,_,GXY,_) -> sec.
    rule:: GXY, completedA(b,a,_,_,GXY,_) -> sec.
    rule:: GXY, completedB(a,b,_,_,GXY,_) -> sec.
    rule:: GXY, completedB(b,a,_,_,GXY,_) -> sec.

    rule:: g^dhx(a,b)^dhy(b,a) -> sec.
    rule:: g^dhx(b,a)^dhy(a,b) -> sec.

    rule:: completedA(a, b, _, g^dhy(b,c), _, _) -> sec.
    rule:: completedA(b, a, _, g^dhy(a,c), _, _) -> sec.

    rule:: completedB(b, a, g^dhx(a,c), _, _, _) -> sec. 
    rule:: completedB(a, b, g^dhx(b,c), _, _, _) -> sec.

    %%%
    %% should fire
    %
    % rule:: completedA(a,a,GXA,GYA,GXYA,SKA), completedB(a,a,GXB,GYB,GXYB,SKB) -> sec.
    % rule:: completedA(a,a,GX,GY,GXY,SK), completedB(a,a,GX,GY,GXY,SK) -> sec.
    % rule:: SK, completedA(a,c,_,_,_,SK) -> sec.
    % rule:: SK, completedB(a,c,_,_,_,SK) -> sec.

