%%
%%  ISO-KE
%%


% Functors:

    % cryptographic primitives
    fun sig/2.
    fun enc/2.

    % participant names 
    fun a/0.
    fun b/0.
    fun c/0.

    % CA
    fun ca/2.

    % constants
    fun g/0.
    fun sec/0.

    % nonces and keys
    fun rA/2.   % rA(X,Y) -- a nonce (exponent) generated by X in role A for the session with Y.
    fun rB/2.   % rB(X,Y) -- a nonce (exponent) generated by X in role B for the session with Y.
    fun k/1.    % k(K) -- private key of X.
    fun pub/1.
    fun key/2.  % key(X,Y) -- a key shared by X and Y.

    fun completedA/3.
    fun completedB/3.


%%%%% AUXILIARY PREDICATES %%%%%%%%%%%%

participant(a).
%participant(b).
participant(c).
honest(a).
%honest(b).

%%
% The following predicates specify which session will be
% taken into consideration.

% roleA(X,Y)  ---  X plays role A with Y plaing role B
% roleA(X,Y)  ---  X plays role A with Y plaing role B
%
roleA(a, a).
% roleA(a, b).
roleA(a, c).
% roleA(b, a).
% roleA(b, c).

roleB(a, a).
%roleB(a, b).
roleB(a, c).
%roleB(b, a).
%roleB(b, c).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The secret:

    query sec.

% Intruder rules

    rule:: sig(_,M) -> M.
    rule:: K, M -> sig(K,M).
    rule:: K, M -> enc(K,M).
    rule:: K, enc(K,M) -> M.

% Modelling CA

    % Honest participants have regitered their public keys properly:
    rule:: ca(A,pub(k(A)))  :- honest(A).

    % A dishonest participant can register anything as his public key:
    rule:: X -> ca(c,X).

% Initial intruder knowledge

    rule:: X             :- participant(X).
    rule:: pub(k(X))     :- participant(X).
    rule:: k(c).
    rule:: rA(c,X)       :- participant(X).
    rule:: rB(c,X)       :- participant(X).
    rule:: key(c,X)      :- participant(X).
    rule:: key(X,c)      :- participant(X).
    rule:: g.

% Protocol Rules (the main session):

    % A->B:
    rule:: 
        A, g^rA(A,B)
    :- 
        roleA(A,B).

    % B->A:
    rule::  
        A, GA ->  B, g^rB(B,A), sig(k(B), (GA, g^rB(B,A), A))
    :- 
        roleB(B,A).

    % A->B:
    rule::  
        B, ca(B,pub(KB)), GB, sig(KB,(g^rA(A,B), GB, A)) 
        ->  sig(k(A),(GB, g^rA(A,B), B)), completedA(A,B,GB^rA(A,B))
    :- roleA(A,B).

    % B comleted:
    rule::  
        ca(A,pub(KA)), sig(KA,(g^rB(B,A), GA, B)) -> completedB(B,A,GA^rB(B,A))
    :- 
        roleB(B,A).

%%%%%%
    
    %%%
    %% should not fire
    %
    
    rule::  K, completedB(a,a,K) -> sec.  
    rule::  K, completedB(a,a,K) -> sec.  

    rule:: completedA(a,a,K), completedB(a,c,K) -> sec.
    rule:: completedA(a,c,K), completedB(a,a,K) -> sec.
    
    rule:: g^rA(a,a)^rB(a,a) -> sec.

    %%%
    %% should fire
    %
    % rule:: completedA(a,a,K), completedB(a,a,K) -> sec.
    % rule::  K, completedA(a,c,K) -> sec.  
    % rule::  K, completedB(a,c,K) -> sec.  
 
