%%
%%  SIGMA-Basic
%%  
%%  H. Krawczyk, SIGMA: the `SIGn-and-MAc' Approach to Authenticated
%%  Diffie-Hellman and its Use in the IDE Protocols, CRYPTIO'03
%%


% Functors:

    % cryptographic primitives
    fun sig/2.
    fun enc/2.

    % CA
    fun ca/2.

    fun mac/2.

    % participant names 
    fun a/0.
    fun b/0.
    fun b1/0.
    fun b2/0.
    fun c/0.

    % constants
    fun g/0.
    fun sec/0.

    % nonces and keys
    fun n/2.    % n(X,Y) -- a nonce (exponent) generated by X as an initiator in the session with Y.
    fun m/2.    % m(X,Y) -- a nonce (exponent) generated by X as a responder in the session with Y.
    fun k/1.    % k(K) -- private key of X.
    fun pub/1.
    fun km/1.   % km(S) -- a key derived from a secret S

    fun completedA/3.
    fun completedB/3.


%%%%% AUXILIARY PREDICATES %%%%%%%%%%%%

participant(a).
 % participant(b).
participant(c).
honest(a).
 % honest(b).
dishonest(c).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The secret:

    query sec.

% Intruder rules

    rule:: sig(_,M) -> M.
    rule:: sig(K,_) -> pub(K).
    rule:: K, M -> sig(K,M).
    rule:: K, M -> enc(K,M).
    rule:: K, enc(K,M) -> M.
    rule:: K, M -> mac(K,M).
    rule:: mac(_,M) -> M.
    rule:: X -> km(X).

% Modelling CA

    % Honest participants have regitered their public keys properly:
    rule:: ca(A,pub(k(A)))  :- honest(A).

    % A dishonest participant can register anything as his public key:
    rule:: X -> ca(C,X)  :- dishonest(C).

% Initial intruder knowledge

    rule:: X             :- participant(X).
    rule:: pub(k(X))     :- participant(X).
    rule:: k(X)          :- dishonest(X).
    % rule:: k(a).
    rule:: g.

% Protocol Rules (the main session):

    % A->B:
    %
    rule::  
        g^n(A,B)
    :- 
        honest(A), participant(B).

    % B->A:
    rule::  
        GX ->  g^m(B,A), B, sig(k(B),(GX,g^m(B,A))), mac(km(GX^m(B,A)), B)
    :- 
        honest(B), participant(A).

    % A->B:
    rule::  
        GY, B, ca(B,pub(KB)), sig(KB,(g^n(A,B), GY)), mac(km(GY^n(A,B)), B) 
            -> A, sig(k(A),(GY,g^n(A,B))), mac(km(GY^n(A,B)), A), completedA(A,B,GY^n(A,B))
    :- 
        honest(A), participant(B).

    % B comleted:
    rule:: 
        A, ca(A,pub(KA)), sig(KA,(g^m(B,A),GX)), mac(km(GX^m(B,A)), A) 
            -> completedB(B,A,GX^m(B,A))
    :- 
        honest(B), participant(A).

%%%%%%
    
    %%%
    %% should not fire
    %
    
     rule::  K, completedA(A,B,K) -> sec  
         :- honest(A), honest(B).
 
     rule::  K, completedB(B,A,K) -> sec  
         :- honest(A), honest(B).
 
     rule:: completedA(A,B,K), completedB(B,C,K) -> sec
         :- honest(A), honest(B), participant(C), A\=C.
 
     rule:: completedA(A,C,K), completedB(B,A,K) -> sec
         :- honest(A), honest(B), participant(C), B\=C.
 
     rule:: g^n(A,B)^m(B,A) -> sec
         :- honest(A), honest(B), A\=B.
   
    %%%
    %% should fire
    %
    % rule:: completedA(a,a,K), completedB(a,a,K) -> sec.
    % rule::  K, completedA(a,c,K) -> sec.  

